// error.hh
// Error handling class
// Kevin Puckett Albrecht
// Rev 12
// 19 March 2000 - 4 May 2001
//
// must define: GLOBAL_ERROR_STRING_SIZE

#ifndef ERROR_HH
#define ERROR_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

// Program Error Codes //
#define EP_INVALID_ARG               "P01: Invalid command line argument"
#define EP_FILE_OPEN_FAILED          "P02: Error opening file"

// Syntax Error Codes //
#define ES_UNDEFINED_TOKEN           "S01: Undefined token"
#define ES_UNDEFINED_IDENTIFIER      "S02: Undefined identifier"
#define ES_UNEXPECTED_TOKEN          "S03: Unexpected token"
#define ES_UNEXPECTED_EOF            "S04: Unexpected end-of-file"
#define ES_MISSING_EOL               "S05: Missing end-of-line"
#define ES_MISSING_ASSIGNMENT_OP     "S06: Missing assignment operator"
#define ES_MISSING_RIGHT_PAREN       "S07: Missing right parenthesis"
#define ES_INVALID_EXPRESSION        "S08: Invalid expression"
#define ES_ALREADY_DECLARED          "S09: Identifier already declared"
#define ES_TYPE_INCOMPATIBLE         "S10: Type is incompatible"
#define ES_CONST_ASSIGNMENT          "S11: Cannot assign a new value to a constant"
#define ES_NESTING_TOO_DEEP          "S12: Nesting too deep"
#define ES_INVALID_PARSE_POINT       "S13: Invalid parsing point"
#define ES_REPEAT_DECLARATIVE        "S14: Only one declarative sequence allowed per program"
#define ES_SUB_ALREADY_DECLARED      "S15: Subroutine already declared"
#define ES_VARIABLE_IN_DECLARATION   "S16: Cannot use variable in its own declaration"
#define ES_INCOMPLETE_COMMENT        "S18: Comment is never closed"

// Runtime Error Codes //
#define ER_CODE_SEGMENT_OVERFLOW     "R01: Code segment overflow"
#define ER_RUNTIME_ERROR             "R02: Runtime error"
#define ER_DIVIDE_BY_ZERO            "R03: Divide by zero"
#define ER_STACK_OVERFLOW            "R04: Runtime stack overflow error"
#define ER_STACK_EMPTY               "R05: Runtime stack empty"
#define ER_INVALID_EXEC_POINT        "R06: Invalid execution point"
#define ER_EXCESSIVE_GOTOS           "R07: Too many gotos were executed"
#define ER_INVALID_ASSIGNMENT        "R08: Invalid assignment"
#define ER_DESTINATION_NOT_DEFINED   "R09: Attempted to goto undefined destination"
//#define ER_VARIABLE_ALREADY_DEFINED  "R10: Variable previously defined"
#define ER_VARIABLE_NOT_DEFINED      "R11: Variable not defined"
#define ER_FILE_ALREADY_OPEN         "R12a: File already open"
#define ER_FILE_NOT_OPEN             "R12b: File not open"
#define ER_ATTEMPT_INPUT_ON_OUTFILE  "R13: Cannot perform input on an output file"
#define ER_ATTEMPT_OUTPUT_ON_INFILE  "R14: Cannot perform output on an input file"

/**** Not yet implemented ****/
#define ES_RAT_OUT_OF_RANGE          "Rational number out of range"
#define ES_INT_OUT_OF_RANGE          "Integer out of range"
#define ES_TOO_MANY_DIGITS           "Too many digits"
#define ES_NOT_IMPLEMENTED           "Feature not yet implemented"

// Effect Codes //
#define EFF_IGNORE 0
#define EFF_RECOVER 1
#define EFF_TERMINATE 2

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <iostream.h>
#include <stdlib.h>

// KLib //
#include "k_string.hh"

// Luck //
#include "common.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

class Error
{
  public:
    Error (char *aHeader, bool abPrintLine);
    ~Error ();

    void Show (int nEffect = 0, char *sError = 0, char *sClarification = 0, char *sMessage = 0);
    static void Verbose (char *s1 = "", char *s2 = "", char *s3 = "", char *s4 = "", bool bLine = true);

  private:
    char *mHeader;
    bool mbPrintLine;
};

//// Constructor ////
Error::Error (char *aHeader, bool abPrintLine)
{
  mbPrintLine = abPrintLine;

  mHeader = new char[GLOBAL_ERROR_STRING_SIZE];
  k_copy (mHeader, aHeader);
}

//// Destructor ////
Error::~Error ()
{
  k_del (mHeader);
}

//// Show ////
void Error::Show (int nEffect, char *sError, char *sClarification, char *sMessage)
{
  gpCommon->nErrors++;

  cerr << "** ";
  if (mbPrintLine) cerr << "Line " << gpCommon->nCurrentLine << ": ";
  cerr << mHeader << " " << sError;
  if (sClarification) cerr << ": " << sClarification;
  if (sMessage) cerr << "\n** (continued...) " << sMessage;

  switch (nEffect)
  {
    case (EFF_IGNORE):
      break;
    case (EFF_RECOVER):
      break;
    case (EFF_TERMINATE):
      cerr << "\n** Fatal Error: Terminating program...\n";
      exit (0);
      break;
    default:
      cerr << "\n** Invalid Error Effect: " << nEffect;
      break;
  }

  cerr << "\n";
}

//// Verbose ////
void Error::Verbose (char *s1, char *s2, char *s3, char *s4, bool bLine)
{
  if (!gpCommon->bVerbose) return;

  cout << "<<";
  if (bLine) cout << "#" << gpCommon->nCurrentLine << ":";
  if (s1) if (strlen(s1) > 0) cout << s1;
  if (s2) if (strlen(s2) > 0) cout << ":" << s2;
  if (s3) if (strlen(s3) > 0) cout << ":" << s3;
  if (s4) if (strlen(s4) > 0) cout << ":" << s4;
  cout << ">>\n";
}

#endif
